/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file post_ejection
 * @author Brendan Bruner
 * @date Feb 12, 2015
 */

#include "system_state_relay.h"
#include "system_states.h"
#include "commands.h"
#include "driver_toolkit.h"

/************************************************************************/
/* Next State Method													*/
/************************************************************************/
static system_state_t *next_state_logic( 	system_state_t *state,
											system_state_relay_t *relay )
{
	static_timer_t	*timer;
	eps_t			*eps;

	timer = &relay->drivers->timer;
	eps = &relay->drivers->eps;

	/* If the static timer hasn't expired, stay in this state. */
	if( timer->is_expired( timer ) == STATIC_TIMER_NOT_EXPIRED )
	{
		return NULL;
	}

	/* If the static timer expired, check the eps' mode. */
	if( timer->is_expired( timer ) == STATIC_TIMER_EXPIRED )
	{
		/* If optimal, go to start up state. */
		if( eps->mode( eps ) == EPS_MODE_OPTIMAL )
		{
			return &relay->start_up;
		}
		/* Otherwise, go to pre charge state. */
		else
		{
			return &relay->pre_charge;
		}
	}

	/* Fail safe, stay in this state if the static timer returned garbage. */
	return NULL;
}

/************************************************************************/
/* Initialization Method												*/
/************************************************************************/
void initialize_post_ejection_state( system_state_t *state )
{	
	state->generic_data =	NULL;
	state->id = 			STATE_POST_EJECTION_ID;

	state->enter_system_state = NO_ENTRY_EXIT_ROUTINE();
	state->exit_system_state = 	NO_ENTRY_EXIT_ROUTINE();

	state->next_system_state = &next_state_logic;

	/* TODO: assign command execution ability */
	state->collect_housekeeping = 				DISABLE_COMMAND_EXECUTION();
	state->transmit_data = 						DISABLE_COMMAND_EXECUTION();
	state->handle_DFGM = 						DISABLE_COMMAND_EXECUTION();
	state->handle_MnLP = 						DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_high_priority = 	DISABLE_COMMAND_EXECUTION();
	state->handle_command_high_priority = 		DISABLE_COMMAND_EXECUTION();
	state->handle_command_low_priority =		DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_low_priority = 	DISABLE_COMMAND_EXECUTION();
}
