/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file power_on
 * @author Brendan Bruner
 * @date Jan 29, 2015
 *
 * Function implementation for the power on state.
 */

#include "system_state_relay.h"
#include "system_states.h"
#include "commands.h"
#include "driver_toolkit.h"

/************************************************************************/
/* Entry / Exit Methods													*/
/************************************************************************/

/************************************************************************/
/* Next State Method													*/
/************************************************************************/
static system_state_t *next_state_logic(	system_state_t *state,
											system_state_relay_t *relay )
{
	ejection_pin_t *eject;

	/* Check if the satellite is ejected from its pod. If it is, then
	 * return the next state (post ejection state) otherwise return
	 * NULL to prevent a state change.
	 */
	eject = &relay->drivers->ejection_pin;
	if( eject->is_ejected( eject ) == EJECTED )
	{
		return &relay->post_ejection;
	}
	return NULL;
}

/************************************************************************/
/* Initialization Method												*/
/************************************************************************/
void initialize_power_on_state( system_state_t *state )
{
	state->generic_data =	NULL;
	state->id = 			STATE_POWER_ON_ID;

	state->enter_system_state = NO_ENTRY_EXIT_ROUTINE();
	state->exit_system_state = 	NO_ENTRY_EXIT_ROUTINE();

	state->next_system_state = &next_state_logic;

	state->collect_housekeeping = 				DISABLE_COMMAND_EXECUTION();
	state->transmit_data = 						DISABLE_COMMAND_EXECUTION();
	state->handle_DFGM = 						DISABLE_COMMAND_EXECUTION();
	state->handle_MnLP = 						DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_high_priority = 	DISABLE_COMMAND_EXECUTION();
	state->handle_command_high_priority = 		DISABLE_COMMAND_EXECUTION();
	state->handle_command_low_priority =		DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_low_priority = 	DISABLE_COMMAND_EXECUTION();
}
