/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file power_safe
 * @author Brendan Bruner
 * @date Jan 29, 2015
 */

#include "system_state_relay.h"
#include "system_states.h"
#include "commands.h"

#ifndef NULL
#define NULL (void *) 0
#endif

/************************************************************************/
/* Entry / Exit Methods													*/
/************************************************************************/
/* No entry / exit routines used by power safe state. */

/************************************************************************/
/* Next State Method													*/
/************************************************************************/
static system_state_t * next_state_from_power_safe( system_state_t *state,
													system_state_relay_t *relay)
{
	eps_t *eps;
	comm_t *comm;

	eps = &relay->drivers->eps;
	comm = &relay->drivers->comm;

	/* If battery is normal or higher and the OBC is being beaconed, go to transmit mode. */
	if( (eps->mode( eps ) == EPS_MODE_NORMAL || eps->mode( eps ) == EPS_MODE_OPTIMAL) &&
		comm->beacon( comm ) == COMM_IS_BEACONED )
	{
		return &relay->transmit;
	}
	/* Else, if battery is optimal, go to detumble. */
	else if( eps->mode( eps ) == EPS_MODE_OPTIMAL )
	{
		return &relay->detumble;
	}
	/* Fail safe, stay in this state. */
	else
	{
		return NULL;
	}
}

/************************************************************************/
/* Initialization Method												*/
/************************************************************************/
void initialize_power_safe_state( system_state_t *state )
{
	state->generic_data =	NULL;
	state->id = 			STATE_POWER_SAFE_ID;

	state->enter_system_state = NO_ENTRY_EXIT_ROUTINE();
	state->exit_system_state = 	NO_ENTRY_EXIT_ROUTINE();

	state->next_system_state = &next_state_from_power_safe;

	state->collect_housekeeping = 				ENABLE_COMMAND_EXECUTION();
	state->transmit_data = 						DISABLE_COMMAND_EXECUTION();
	state->handle_DFGM = 						DISABLE_COMMAND_EXECUTION();
	state->handle_MnLP = 						DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_high_priority = 	ENABLE_COMMAND_EXECUTION();
	state->handle_command_high_priority = 		ENABLE_COMMAND_EXECUTION();
	state->handle_command_low_priority =		DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_low_priority = 	DISABLE_COMMAND_EXECUTION();
}
