/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file pre_charge
 * @author Brendan Bruner
 * @date Mar 2, 2015
 */

#include "system_state_relay.h"
#include "system_states.h"
#include "commands.h"

/************************************************************************/
/* Entry / Exit Methods													*/
/************************************************************************/
static void entry_method(	system_state_t *state,
							system_state_relay_t *relay )
{
	/* TODO: entry method */
}

static void exit_method(	system_state_t *state,
							system_state_relay_t *relay )
{
	/* TODO: entry method */
}

/************************************************************************/
/* Next State Method													*/
/************************************************************************/
static system_state_t *next_state_logic( 	system_state_t *state,
											system_state_relay_t *relay )
{
	eps_t *eps;

	eps = &relay->drivers->eps;

	/* Refresh EPS hk structure to update battery voltage */
	/* TODO Possibly modify this to update EPS vi structure instead */
	eps->hk_refresh( eps );

	/* If battery is optimal, go to start up state. */
	if( eps->mode( eps ) == EPS_MODE_OPTIMAL )
	{
		return &relay->start_up;
	}
	else
	/* Otherwise, stay in this state so battery can charge. */
	{
		return NULL;
	}
}

/************************************************************************/
/* Initialization Method												*/
/************************************************************************/
void initialize_pre_charge_state( system_state_t *state )
{	
	state->generic_data =	NULL;
	state->id = 			STATE_PRE_CHARGE_ID;

	state->enter_system_state = NO_ENTRY_EXIT_ROUTINE(); /* TODO: assign entry */
	state->exit_system_state = 	NO_ENTRY_EXIT_ROUTINE(); /* exit routines	   */

	state->next_system_state = &next_state_logic;/* TODO: assign next state logic */

	/* TODO: assign command execution ability */
	state->collect_housekeeping = 				DISABLE_COMMAND_EXECUTION();
	state->transmit_data = 						DISABLE_COMMAND_EXECUTION();
	state->handle_DFGM = 						DISABLE_COMMAND_EXECUTION();
	state->handle_MnLP = 						DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_high_priority = 	DISABLE_COMMAND_EXECUTION();
	state->handle_command_high_priority = 		DISABLE_COMMAND_EXECUTION();
	state->handle_command_low_priority =		DISABLE_COMMAND_EXECUTION();
	state->handle_command_very_low_priority = 	DISABLE_COMMAND_EXECUTION();
}
