/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file system_state_generic
 * @author Brendan Bruner
 * @date Jan 29, 2015
 */

#include "system_states.h"
#include "system_state_relay.h"
#include "commands.h"

/************************************************************************/
/* Global Defines														*/
/************************************************************************/
char const * const state_id_to_name[] = {	"power on", "post ejection",
											"pre charge", "start up",
											"power safe", "science",
											"detumble", "transmit", ""	};


/************************************************************************/
/* Method implementations 												*/
/************************************************************************/
system_state_id_t query_system_state_id( system_state_t *state )
{
	if( state == NULL )
	{
		return STATE_NULL;
	}
	return state->id;
}

/************************************************************************/
/* Generic implementations of virtual methods							*/
/************************************************************************/
void state_enables_execute( system_state_t *state, command_t *command )
{
	if( command == NULL )
	{
		return;
	}
	execute_command( command );
}

void state_disables_execute( system_state_t *state, command_t *command)
{
	return;
}

void state_no_entry_no_exit( system_state_t *state, system_state_relay_t *relay )
{
	return;
}

system_state_t *never_exit_state(	system_state_t *state,
									system_state_relay_t *relay )
{
	return NULL;
}
