/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file uart
 * @author Brendan Bruner
 * @date Mar 28, 2015
 */
#ifndef INCLUDE_UART_H_
#define INCLUDE_UART_H_

#include <stdint.h>

/**
 * @brief
 * 		Initialize UART2 of lpc board for rx and tx
 * @details
 * 		Initialize UART2 of lpc board for rx and tx. Successive calls after
 * 		the first call are ignored.
 */
void serial2_begin( void );

/**
 * @brief
 * 		Put data on the serial line.
 * @details
 * 		Put data on the serial line.
 * @param data
 * 		A pointer to the data to put on the line.
 * @param size
 * 		The number of bytes in the array pointed to by data.
 * @returns
 * 		The number of bytes put on the line.
 */
uint32_t serial2_send( uint8_t const *data, uint8_t size );

/**
 * @brief
 * 		Read data off the serial line.
 * @details
 * 		Received data on the serial line is put into a buffer. This method
 * 		copies that buffer into the first argument data.
 * @param data
 * 		A pointer to the array where data will be copied into.
 * @param size
 * 		The number of bytes to copy into data array.
 * @returns
 * 		The number of bytes copied into the data array.
 */
uint32_t serial2_read( uint8_t *data, uint8_t size );

/**
 * @brief
 * 		Poll the number of bytes which has received from the serial line
 * 		and buffered.
 * @details
 * 		Poll the number of bytes which has received from the serial line
 * 		and buffered.
 * @returns
 * 		The number of bytes which have been buffered off the serial lien.
 */
uint32_t serial2_available( void );

void serial2_lock( void );
void serial2_unlock( void );
void serial2_reset( void );

#endif /* INCLUDE_UART_H_ */
