/*

 * Copyright (C) 2015  Brendan Bruner, Oleg Oleynikov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca

*
 * @file adcs_arduino_remote
 * @author Brendan Bruner
 * @author Oleg Oleynikov
 * @date Mar 04, 2015
*/


#include "adcs.h"
#include "i2c.h"
#include <stdlib.h>
#include <time.h>
#include <string.h>


I2C_M_SETUP_Type MasterConfig;
uint32_t temp;
/***********************************************************************
 Virtual method implementations
***********************************************************************/
static adcs_mode_t mode_mock_up( adcs_t *adcs )
{
	i2c_lock( );
	I2CSend(MasterConfig, temp, 255, NULL, 0);
	adcs_mode_t mode = (adcs_mode_t)I2CRecieve(MasterConfig, temp, 1 );
	i2c_unlock( );

	return mode;
}

/* General telecommands*/
static void adcs_reset(adcs_t *self, adcs_reset_t *reset_type )
{
	I2CSend(MasterConfig, temp, 1, (uint8_t *) reset_type, sizeof(adcs_reset_t));
}

static void adcs_set_unix_time(adcs_t *self, adcs_unix_time_t *time)
{
	I2CSend(MasterConfig, temp, 2, (uint8_t *) time, sizeof(adcs_unix_time_t));
}

static void adcs_set_run_mode(adcs_t *self, adcs_run_mode_t *run_mode)
{
	I2CSend(MasterConfig, temp, 3, (uint8_t *) run_mode, sizeof(adcs_run_mode_t));
}

static void adcs_select_logged_data(adcs_t *self, adcs_logged_data_t *logged_data)
{

	I2CSend(MasterConfig, temp, 4, (uint8_t *) logged_data, sizeof(adcs_logged_data_t));
}

static void adcs_select_element_power_control(adcs_t *self, adcs_power_control_t *power_state)
{
	I2CSend(MasterConfig, temp, 5, (uint8_t *) power_state, sizeof(adcs_power_control_t));
}

static void adcs_deploy_magnetometer_boom(adcs_t *self, adcs_deploy_magnetometer_boom_timeout_t *timeout)
{

	I2CSend(MasterConfig, temp, 6, (uint8_t *) timeout, sizeof(adcs_deploy_magnetometer_boom_timeout_t));
}

static void adcs_trigger_adcs_loop(adcs_t *self)
{

	I2CSend(MasterConfig, temp, 7, NULL, 0);
}

static void adcs_set_attitude_estimation_mode(adcs_t *self, adcs_attitude_estimation_mode_t *estimation_mode)
{
	I2CSend(MasterConfig, temp, 17, (uint8_t *) estimation_mode, sizeof(adcs_attitude_estimation_mode_t));

}

static void adcs_set_attitude_control_mode(adcs_t *self, adcs_attitude_control_mode_t *control_mode)
{
	I2CSend(MasterConfig, temp, 18, (uint8_t *) control_mode, sizeof(adcs_attitude_control_mode_t));

}

static void adcs_set_commanded_attitude_angles(adcs_t *self, adcs_commanded_attitude_angles_t *angles)
{
	I2CSend(MasterConfig, temp, 19, (uint8_t *) angles, sizeof(adcs_commanded_attitude_angles_t));

}

static void adcs_set_wheel_speed(adcs_t *self, adcs_wheel_speed_t *speed)
{
	I2CSend(MasterConfig, temp, 32, (uint8_t *) speed, sizeof(adcs_wheel_speed_t));
}

static void adcs_set_magnetorquer_output(adcs_t *self, adcs_magnetorquer_output_t *output)
{
	I2CSend(MasterConfig, temp, 33, (uint8_t *) output, sizeof(adcs_magnetorquer_output_t));
}

static void adcs_set_sgp4_orbit_parameters(adcs_t *self, adcs_sgp4_orbit_parameters_t *params)
{
	I2CSend(MasterConfig, temp, 64, (uint8_t *) params, sizeof(adcs_sgp4_orbit_parameters_t));
}

static void adcs_set_curr_config(adcs_t *self, adcs_curr_config_t *config)
{
	I2CSend(MasterConfig, temp, 80, (uint8_t *) config, sizeof(adcs_curr_config_t));
}

static void adcs_set_magnetorquer_config(adcs_t *self, adcs_magnetorquer_config_t *magnetorquer_config)
{
	I2CSend(MasterConfig, temp, 81, (uint8_t *) magnetorquer_config, sizeof(adcs_magnetorquer_config_t));
}
static void adcs_set_wheel_config(adcs_t *self, adcs_wheel_config_t *wheel_config)
{
	I2CSend(MasterConfig, temp, 82, (uint8_t *) wheel_config, sizeof(adcs_wheel_config_t));
}
static void adcs_set_css_config(adcs_t *self, adcs_css_config_t *css_config)
{
	I2CSend(MasterConfig, temp, 83, (uint8_t *) css_config, sizeof(adcs_css_config_t));
}
static void adcs_set_sun_sensor_config(adcs_t *self, adcs_sun_sensor_config_t *sun_sensor_config)
{
	I2CSend(MasterConfig, temp, 84, (uint8_t *) sun_sensor_config, sizeof(adcs_sun_sensor_config_t));
}
static void adcs_set_nadir_sensor_config(adcs_t *self, adcs_nadir_sensor_config_t *nadir_sensor_config)
{
	I2CSend(MasterConfig, temp, 85, (uint8_t *) nadir_sensor_config, sizeof(adcs_nadir_sensor_config_t));
}
static void adcs_set_magnetometer_config(adcs_t *self, adcs_magnetometer_config_t *magnetometer_config)
{
	I2CSend(MasterConfig, temp, 86, (uint8_t *) magnetometer_config, sizeof(adcs_magnetometer_config_t));
}
static void adcs_set_rate_sensor_config(adcs_t *self, adcs_rate_sensor_config_t *rate_sensor_config)
{
	I2CSend(MasterConfig, temp, 87, (uint8_t *) rate_sensor_config, sizeof(adcs_rate_sensor_config_t));
}
static void adcs_set_detumbling_control(adcs_t *self, adcs_detumbling_control_t *detumbling_control)
{
	I2CSend(MasterConfig, temp, 88, (uint8_t *) detumbling_control, sizeof(adcs_detumbling_control_t));
}
static void adcs_set_Y_momentum_control(adcs_t *self, adcs_Y_momentum_control_t *Y_momentum_control_control)
{
	I2CSend(MasterConfig, temp, 89, (uint8_t *) Y_momentum_control_control, sizeof(adcs_Y_momentum_control_t));
}
static void adcs_set_moment_of_inertia(adcs_t *self, adcs_moment_of_inertia_t *moment_of_inertia)
{
	I2CSend(MasterConfig, temp, 90, (uint8_t *) moment_of_inertia, sizeof(adcs_moment_of_inertia_t));
}
static void adcs_set_estimation_parameters(adcs_t *self, adcs_estimation_parameters_t *estimation_parameters)
{
	I2CSend(MasterConfig, temp, 91, (uint8_t *) estimation_parameters, sizeof(adcs_estimation_parameters_t));
}

static void adcs_save_config(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 100, NULL, 0);

}

static void adcs_save_orbit_params(adcs_t *self)
{

	I2CSend(MasterConfig, temp, 101, NULL, 0);
}

static void adcs_set_start_up_mode(adcs_t *self, adcs_start_up_mode_t *start_up_mode)
{
	I2CSend(MasterConfig, temp, 102, (uint8_t *) start_up_mode, sizeof(adcs_start_up_mode_t));

}

static void adcs_capture_and_save_image(adcs_t *self, adcs_capture_and_save_image_t *settings)
{

	I2CSend(MasterConfig, temp, 110, (uint8_t *) settings, sizeof(adcs_capture_and_save_image_t));
}

static void adcs_reset_file_list(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 114, NULL, 0);

}

static void adcs_advance_file_list_index(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 115, NULL, 0);

}

static void adcs_init_download(adcs_t *self, char *filename, int length)
{
	if (length > 13) {
		return;
	}
	I2CSend(MasterConfig, temp, 116, (uint8_t *) filename, strlen(filename));

}

static void adcs_advance_file_read_pointer(adcs_t *self, adcs_next_block_num_to_read_t *block_number)
{
	I2CSend(MasterConfig, temp, 117, (uint8_t *) block_number, sizeof(adcs_next_block_num_to_read_t));

}

static void adcs_erase_file(adcs_t *self, char *filename, int length)
{
	if (length > 13) {
		return;
	}
	I2CSend(MasterConfig, temp, 118, (uint8_t *) filename, strlen(filename));

}

static void adcs_erase_all_files(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 119, NULL, 0);

}

static void adcs_set_boot_index(adcs_t *self, adcs_boot_index_t *index)
{
	I2CSend(MasterConfig, temp, 120, (uint8_t *) index, sizeof(adcs_boot_index_t));

}

static void adcs_erase_program(adcs_t *self, adcs_boot_index_t *index)
{
	I2CSend(MasterConfig, temp, 121, (uint8_t *) index, sizeof(adcs_boot_index_t));

}

static void adcs_upload_program_block(adcs_t *self, adcs_upload_program_block_t *program_block)
{
	/* TODO: sizeof( adcs_upload_program_block_t ) = 259 not 255 */
	I2CSend(MasterConfig, temp, 122, (uint8_t *) program_block, 255);

}

static void adcs_finalize_program_upload(adcs_t *self, adcs_finalize_program_upload_t *finalize_upload_command)
{
	I2CSend(MasterConfig, temp, 123, (uint8_t *) finalize_upload_command, sizeof(adcs_finalize_program_upload_t));

}

/*General telemetry*/
static adcs_identification_t* adcs_send_identification(adcs_t *self)
{

	I2CSend(MasterConfig, temp, 128, NULL, 0);
	adcs_identification_t* id = (adcs_identification_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_identification_t));
	return id;
}

static adcs_comm_status_t* adcs_send_comm_status(adcs_t *self)
{
/*	 keep track of this while executing*/
	I2CSend(MasterConfig, temp, 130, NULL, 0);
	adcs_comm_status_t* comm_status = (adcs_comm_status_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_comm_status_t));
	return comm_status;
}

static adcs_ack_t* adcs_send_ack(adcs_t *self)
{
/*	 keep track of it while executing*/
	I2CSend(MasterConfig, temp, 131, NULL, 0);
	adcs_ack_t* ack = (adcs_ack_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_ack_t));
	return ack;
}

static adcs_reset_cause_t* adcs_send_reset_cause(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 133, NULL, 0);
	adcs_reset_cause_t* reset_cause = (adcs_reset_cause_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_reset_cause_t));

	return reset_cause;
}

static adcs_power_control_t* adcs_send_selected_element_power_control(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 134, NULL, 0);
	adcs_power_control_t* power_control = (adcs_power_control_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_power_control_t));
	return power_control;
}

static adcs_power_and_temperature_meas_t* adcs_send_power_and_temperature_meas(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 135, NULL, 0);
	adcs_power_and_temperature_meas_t* power_and_temperature_meas = (adcs_power_and_temperature_meas_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_power_and_temperature_meas_t));
	return power_and_temperature_meas;
}

static adcs_cubesense_current_measurements_t* adcs_send_cubesense_current_measurement(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 173, NULL, 0);
	adcs_cubesense_current_measurements_t* cubesense_current_measurements = (adcs_cubesense_current_measurements_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_cubesense_current_measurements_t));
	return cubesense_current_measurements;
}
static adcs_cubecontrol_current_measurements_t* adcs_send_cubecontrol_current_measurements(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 174, NULL, 0);
	adcs_cubecontrol_current_measurements_t* cubecontrol_current_measurements = (adcs_cubecontrol_current_measurements_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_cubecontrol_current_measurements_t));
	return cubecontrol_current_measurements;
}
static adcs_peripheral_current_and_temperature_meas_t* adcs_send_peripheral_current_and_temperature_meas(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 175, NULL, 0);
	adcs_peripheral_current_and_temperature_meas_t* peripheral_current_and_temperature_meas = (adcs_peripheral_current_and_temperature_meas_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_peripheral_current_and_temperature_meas_t));
	return peripheral_current_and_temperature_meas;
}

static adcs_state_t* adcs_send_state(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 136, NULL, 0);
	adcs_state_t* state = (adcs_state_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_state_t));
	return state;
}

static adcs_unix_time_t* adcs_send_curr_time(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 143, NULL, 0);
	adcs_unix_time_t* curr_time = (adcs_unix_time_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_unix_time_t));
	return curr_time;
}
static adcs_curr_state_t* adcs_send_curr_state(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 144, NULL, 0);
	adcs_curr_state_t* curr_state = (adcs_curr_state_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_curr_state_t));
	return curr_state;
}
static adcs_estimated_attitude_angles_t* adcs_send_estimated_attitude_angles(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 145, NULL, 0);
	adcs_estimated_attitude_angles_t* estimated_attitude_angles = (adcs_estimated_attitude_angles_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_estimated_attitude_angles_t));
	return estimated_attitude_angles;
}
static adcs_estimated_angular_rates_t* adcs_send_estimated_angular_rates(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 146, NULL, 0);
	adcs_estimated_angular_rates_t* estimated_angular_rates = (adcs_estimated_angular_rates_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_estimated_angular_rates_t));
	return estimated_angular_rates;
}
static adcs_satellite_velocity_t* adcs_send_satellite_velocity(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 148, NULL, 0);
	adcs_satellite_velocity_t* satellite_velocity = (adcs_satellite_velocity_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_satellite_velocity_t));
	return satellite_velocity;
}
static adcs_satellite_position_t* adcs_send_satellite_position(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 147, NULL, 0);
	adcs_satellite_position_t* satellite_position = (adcs_satellite_position_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_satellite_position_t));
	return satellite_position;
}

static adcs_measurements_t* adcs_send_measurements(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 137, NULL, 0);
	adcs_measurements_t* measurements = (adcs_measurements_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_measurements_t));
	return measurements;
}

static adcs_magnetic_field_vector_t* adcs_send_magnetic_field_vector(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 149, NULL, 0);
	adcs_magnetic_field_vector_t* magnetic_field_vector = (adcs_magnetic_field_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_magnetic_field_vector_t));
	return magnetic_field_vector;
}
static adcs_coarse_sun_vector_t* adcs_send_coarse_sun_vector(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 150, NULL, 0);
	adcs_coarse_sun_vector_t* coarse_sun_vector = (adcs_coarse_sun_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_coarse_sun_vector_t));
	return coarse_sun_vector;
}
static adcs_fine_sun_vector_t* adcs_send_fine_sun_vector(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 151, NULL, 0);
	adcs_fine_sun_vector_t* fine_sun_vector = (adcs_fine_sun_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_fine_sun_vector_t));
	return fine_sun_vector;
}
static adcs_nadir_vector_t* adcs_send_nadir_vector(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 152, NULL, 0);
	adcs_nadir_vector_t* nadir_vector = (adcs_nadir_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_nadir_vector_t));
	return nadir_vector;
}
static adcs_rate_sensor_rates_t* adcs_send_rate_sensor_rates(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 153, NULL, 0);
	adcs_rate_sensor_rates_t* rate_sensor_rates = (adcs_rate_sensor_rates_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_rate_sensor_rates_t));
	return rate_sensor_rates;
}
static adcs_wheel_speed_meas_t* adcs_send_wheel_speed_meas(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 154, NULL, 0);
	adcs_wheel_speed_meas_t* wheel_speed_meas = (adcs_wheel_speed_meas_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_wheel_speed_meas_t));
	return wheel_speed_meas;
}

static adcs_actuator_command_t* adcs_send_actuator_command(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 138, NULL, 0);
	adcs_actuator_command_t* actuator_command = (adcs_actuator_command_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_actuator_command_t));
	return actuator_command;
}

static adcs_magnetorquer_command_t* adcs_send_magnetorquer_command(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 155, NULL, 0);
	adcs_magnetorquer_command_t* magnetorquer_command = (adcs_magnetorquer_command_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_magnetorquer_command_t));
	return magnetorquer_command;
}
static adcs_wheel_speed_command_t* adcs_send_wheel_speed_command(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 156, NULL, 0);
	adcs_wheel_speed_command_t* wheel_speed_command = (adcs_wheel_speed_command_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_wheel_speed_command_t));
	return wheel_speed_command;
}

static adcs_raw_sensor_measurements_t* adcs_send_raw_sensor_measurements(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 139, NULL, 0);
	adcs_raw_sensor_measurements_t* raw_sensor_measurements = (adcs_raw_sensor_measurements_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_sensor_measurements_t));
	return raw_sensor_measurements;
}

static adcs_raw_nadir_sensor_t* adcs_send_raw_nadir_sensor(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 164, NULL, 0);
	adcs_raw_nadir_sensor_t* raw_nadir_sensor = (adcs_raw_nadir_sensor_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_nadir_sensor_t));
	return raw_nadir_sensor;
}
static adcs_raw_sun_sensor_t* adcs_send_raw_sun_sensor(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 165, NULL, 0);
	adcs_raw_sun_sensor_t* raw_sun_sensor = (adcs_raw_sun_sensor_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_sun_sensor_t));
	return raw_sun_sensor;
}
static adcs_raw_css_t* adcs_send_raw_css(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 166, NULL, 0);
	adcs_raw_css_t* raw_css = (adcs_raw_css_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_css_t));
	return raw_css;
}
static adcs_raw_magnetometer_t* adcs_send_raw_magnetometer(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 167, NULL, 0);
	adcs_raw_magnetometer_t* raw_magnetometer = (adcs_raw_magnetometer_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_magnetometer_t));
	return raw_magnetometer;
}
static adcs_raw_GPS_status_t* adcs_send_raw_GPS_status(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 168, NULL, 0);
	adcs_raw_GPS_status_t* raw_GPS_status = (adcs_raw_GPS_status_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_GPS_status_t));
	return raw_GPS_status;
}
static adcs_raw_GPS_time_t* adcs_send_raw_GPS_time(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 169, NULL, 0);
	adcs_raw_GPS_time_t* raw_GPS_time = (adcs_raw_GPS_time_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_GPS_time_t));
	return raw_GPS_time;
}
static adcs_raw_GPS_x_t* adcs_send_raw_GPS_x(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 170, NULL, 0);
	adcs_raw_GPS_x_t* raw_GPS_x = (adcs_raw_GPS_x_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_GPS_x_t));
	return raw_GPS_x;
}
static adcs_raw_GPS_y_t* adcs_send_raw_GPS_y(adcs_t *self)
{
	 /*generated*/
	I2CSend(MasterConfig, temp, 171, NULL, 0);
	adcs_raw_GPS_y_t* raw_GPS_y = (adcs_raw_GPS_y_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_GPS_y_t));
	return raw_GPS_y;
}
static adcs_raw_GPS_z_t* adcs_send_raw_GPS_z(adcs_t *self)
{
	/* generated*/
	I2CSend(MasterConfig, temp, 172, NULL, 0);
	adcs_raw_GPS_z_t* raw_GPS_z = (adcs_raw_GPS_z_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_raw_GPS_z_t));
	return raw_GPS_z;
}

static adcs_estimation_data_t* adcs_send_estimation_data(adcs_t *self)
{
/*	 generated*/
	I2CSend(MasterConfig, temp, 140, NULL, 0);
	adcs_estimation_data_t* estimation_data = (adcs_estimation_data_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_estimation_data_t));
	return estimation_data;
}

static adcs_IGRF_modelled_magnetic_field_vector_t* adcs_send_IGRF_modelled_magnetic_field_vector(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 157, NULL, 0);
	adcs_IGRF_modelled_magnetic_field_vector_t* IGRF_modelled_magnetic_field_vector = (adcs_IGRF_modelled_magnetic_field_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_IGRF_modelled_magnetic_field_vector_t));
	return IGRF_modelled_magnetic_field_vector;
}
static adcs_modelled_sun_vector_t* adcs_send_modelled_sun_vector(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 158, NULL, 0);
	adcs_modelled_sun_vector_t* modelled_sun_vector = (adcs_modelled_sun_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_modelled_sun_vector_t));
	return modelled_sun_vector;
}
static adcs_estimated_gyro_bias_t* adcs_send_estimated_gyro_bias(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 159, NULL, 0);
	adcs_estimated_gyro_bias_t* estimated_gyro_bias = (adcs_estimated_gyro_bias_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_estimated_gyro_bias_t));
	return estimated_gyro_bias;
}
static adcs_estimated_innovation_vector_t* adcs_send_estimated_innovation_vector(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 160, NULL, 0);
	adcs_estimated_innovation_vector_t* estimated_innovation_vector = (adcs_estimated_innovation_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_estimated_innovation_vector_t));
	return estimated_innovation_vector;
}
static adcs_quaternion_error_vector_t* adcs_send_quaternion_error_vector(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 161, NULL, 0);
	adcs_quaternion_error_vector_t* quaternion_error_vector = (adcs_quaternion_error_vector_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_quaternion_error_vector_t));
	return quaternion_error_vector;
}
static adcs_quaternion_covariance_t* adcs_send_quaternion_covariance(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 162, NULL, 0);
	adcs_quaternion_covariance_t* quaternion_covariance = (adcs_quaternion_covariance_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_quaternion_covariance_t));
	return quaternion_covariance;
}
static adcs_angular_rate_covariance_t* adcs_send_angular_rate_covariance(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 163, NULL, 0);
	adcs_angular_rate_covariance_t* angular_rate_covariance = (adcs_angular_rate_covariance_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_angular_rate_covariance_t));
	return angular_rate_covariance;
}

static adcs_ACP_execution_state_t* adcs_send_ACP_execution_state(adcs_t *self)
{
//	 have to keep track of this in the model
	I2CSend(MasterConfig, temp, 189, NULL, 0);
	adcs_ACP_execution_state_t* ACP_execution_state = (adcs_ACP_execution_state_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_ACP_execution_state_t));
	return ACP_execution_state;
}

static adcs_ACP_execution_times_t* adcs_send_ACP_execution_times(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 190, NULL, 0);
	adcs_ACP_execution_times_t* ACP_execution_times = (adcs_ACP_execution_times_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_ACP_execution_times_t));
	return ACP_execution_times;
}

static adcs_sgp4_orbit_parameters_t* adcs_send_sgp4_orbit_parameters(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 191, NULL, 0);
	adcs_sgp4_orbit_parameters_t* sgp4_orbit_parameters = (adcs_sgp4_orbit_parameters_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_sgp4_orbit_parameters_t));
	return sgp4_orbit_parameters;
}

static adcs_curr_config_t* adcs_send_curr_config(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 192, NULL, 0);
	adcs_curr_config_t* curr_config = (adcs_curr_config_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_curr_config_t));
	return curr_config;
}

static adcs_EDAC_and_latchup_counters_t* adcs_send_EDAC_and_latchup_counters(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 193, NULL, 0);
	adcs_EDAC_and_latchup_counters_t* EDAC_and_latchup_counters = (adcs_EDAC_and_latchup_counters_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_EDAC_and_latchup_counters_t));
	return EDAC_and_latchup_counters;
}

static adcs_start_up_mode_t* adcs_send_start_up_mode(adcs_t *self)
{
	I2CSend(MasterConfig, temp, 194, NULL, 0);
	adcs_start_up_mode_t* start_up_mode = (adcs_start_up_mode_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_start_up_mode_t));
	return start_up_mode;
}

static adcs_image_capture_and_save_operations_statuses_t* adcs_send_image_capture_and_save_operations_statuses(adcs_t *self)
{
//	 generated
	I2CSend(MasterConfig, temp, 230, NULL, 0);
	adcs_image_capture_and_save_operations_statuses_t* image_capture_and_save_operations_statuses = (adcs_image_capture_and_save_operations_statuses_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_image_capture_and_save_operations_statuses_t));
	return image_capture_and_save_operations_statuses;
}

static adcs_file_info_t* adcs_send_file_info(adcs_t *self)
{
//	 keep track of it in the model
	I2CSend(MasterConfig, temp, 240, NULL, 0);
	adcs_file_info_t* file_info = (adcs_file_info_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_file_info_t));
	return file_info;
}

static adcs_file_block_CRC_t* adcs_send_file_block_CRC(adcs_t *self)
{
//	 keep track of it in the model
	I2CSend(MasterConfig, temp, 241, NULL, 0);
	adcs_file_block_CRC_t* file_block_CRC = (adcs_file_block_CRC_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_file_block_CRC_t));
	return file_block_CRC;
}

//static unsigned char block2048[8*256];
static void* adcs_send_data_block(adcs_t *self)
{
//	 keep track of it during execution
	I2CSend(MasterConfig, temp, 242, NULL, 0);
	void* file_info = I2CRecieve(MasterConfig, temp, 256);
	return file_info;
}

static adcs_uploaded_program_status_t* adcs_send_uploaded_program_status(adcs_t *self)
{
//	 keep track of it during execution
	I2CSend(MasterConfig, temp, 250, NULL, 0);
	adcs_uploaded_program_status_t* uploaded_program_status = (adcs_uploaded_program_status_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_uploaded_program_status_t));
	return uploaded_program_status;
}

static adcs_flash_program_list_t* adcs_send_flash_program_list(adcs_t *self)
{
//	 keep track of it during execution
	I2CSend(MasterConfig, temp, 251, NULL, 0);
	adcs_flash_program_list_t* flash_program_list = (adcs_flash_program_list_t*)I2CRecieve(MasterConfig, temp, sizeof(adcs_flash_program_list_t));
	return flash_program_list;
}

/***********************************************************************
 Initialization
***********************************************************************/
void initialize_adcs_arduino_remote( adcs_t *adcs )
{
	adcs->data = (void *) 0;
	adcs->reset_count = 0;
	adcs->mode = &mode_mock_up;
	adcs->adcs_reset = &adcs_reset;
	adcs->adcs_set_unix_time = &adcs_set_unix_time;
	adcs->adcs_set_run_mode = &adcs_set_run_mode;
	adcs->adcs_select_logged_data = &adcs_select_logged_data;
	adcs->adcs_select_element_power_control = &adcs_select_element_power_control;
	adcs->adcs_deploy_magnetometer_boom = &adcs_deploy_magnetometer_boom;
	adcs->adcs_trigger_adcs_loop = &adcs_trigger_adcs_loop;
	adcs->adcs_set_attitude_estimation_mode = &adcs_set_attitude_estimation_mode;
	adcs->adcs_set_attitude_control_mode = &adcs_set_attitude_control_mode;
	adcs->adcs_set_commanded_attitude_angles = &adcs_set_commanded_attitude_angles;
	adcs->adcs_set_wheel_speed = &adcs_set_wheel_speed;
	adcs->adcs_set_magnetorquer_output = &adcs_set_magnetorquer_output;
	adcs->adcs_set_sgp4_orbit_parameters = &adcs_set_sgp4_orbit_parameters;
	adcs->adcs_set_curr_config = &adcs_set_curr_config;
	adcs->adcs_set_magnetorquer_config = &adcs_set_magnetorquer_config;
	adcs->adcs_set_wheel_config = &adcs_set_wheel_config;
	adcs->adcs_set_css_config = &adcs_set_css_config;
	adcs->adcs_set_sun_sensor_config = &adcs_set_sun_sensor_config;
	adcs->adcs_set_nadir_sensor_config = &adcs_set_nadir_sensor_config;
	adcs->adcs_set_magnetometer_config = &adcs_set_magnetometer_config;
	adcs->adcs_set_rate_sensor_config = &adcs_set_rate_sensor_config;
	adcs->adcs_set_detumbling_control = &adcs_set_detumbling_control;
	adcs->adcs_set_Y_momentum_control = &adcs_set_Y_momentum_control;
	adcs->adcs_set_moment_of_inertia = &adcs_set_moment_of_inertia;
	adcs->adcs_set_estimation_parameters = &adcs_set_estimation_parameters;
	adcs->adcs_save_config = &adcs_save_config;
	adcs->adcs_save_orbit_params = &adcs_save_orbit_params;
	adcs->adcs_set_start_up_mode = &adcs_set_start_up_mode;
	adcs->adcs_capture_and_save_image = &adcs_capture_and_save_image;
	adcs->adcs_reset_file_list = &adcs_reset_file_list;
	adcs->adcs_advance_file_list_index = &adcs_advance_file_list_index;
	adcs->adcs_init_download = &adcs_init_download;
	adcs->adcs_advance_file_read_pointer = &adcs_advance_file_read_pointer;
	adcs->adcs_erase_file = &adcs_erase_file;
	adcs->adcs_erase_all_files = &adcs_erase_all_files;
	adcs->adcs_set_boot_index = &adcs_set_boot_index;
	adcs->adcs_erase_program = &adcs_erase_program;
	adcs->adcs_upload_program_block = &adcs_upload_program_block;
	adcs->adcs_finalize_program_upload = &adcs_finalize_program_upload;
	adcs->adcs_send_identification = &adcs_send_identification;
	adcs->adcs_send_comm_status = &adcs_send_comm_status;
	adcs->adcs_send_ack = &adcs_send_ack;
	adcs->adcs_send_reset_cause = &adcs_send_reset_cause;
	adcs->adcs_send_selected_element_power_control = &adcs_send_selected_element_power_control;
	adcs->adcs_send_power_and_temperature_meas = &adcs_send_power_and_temperature_meas;
	adcs->adcs_send_cubesense_current_measurement = &adcs_send_cubesense_current_measurement;
	adcs->adcs_send_cubecontrol_current_measurements = &adcs_send_cubecontrol_current_measurements;
	adcs->adcs_send_peripheral_current_and_temperature_meas = &adcs_send_peripheral_current_and_temperature_meas;
	adcs->adcs_send_state = &adcs_send_state;
	adcs->adcs_send_curr_time = &adcs_send_curr_time;
	adcs->adcs_send_curr_state = &adcs_send_curr_state;
	adcs->adcs_send_estimated_attitude_angles = &adcs_send_estimated_attitude_angles;
	adcs->adcs_send_estimated_angular_rates = &adcs_send_estimated_angular_rates;
	adcs->adcs_send_satellite_velocity = &adcs_send_satellite_velocity;
	adcs->adcs_send_satellite_position = &adcs_send_satellite_position;
	adcs->adcs_send_measurements = &adcs_send_measurements;
	adcs->adcs_send_magnetic_field_vector = &adcs_send_magnetic_field_vector;
	adcs->adcs_send_coarse_sun_vector = &adcs_send_coarse_sun_vector;
	adcs->adcs_send_fine_sun_vector = &adcs_send_fine_sun_vector;
	adcs->adcs_send_nadir_vector = &adcs_send_nadir_vector;
	adcs->adcs_send_rate_sensor_rates = &adcs_send_rate_sensor_rates;
	adcs->adcs_send_wheel_speed_meas = &adcs_send_wheel_speed_meas;
	adcs->adcs_send_actuator_command = &adcs_send_actuator_command;
	adcs->adcs_send_magnetorquer_command = &adcs_send_magnetorquer_command;
	adcs->adcs_send_wheel_speed_command = &adcs_send_wheel_speed_command;
	adcs->adcs_send_raw_sensor_measurements = &adcs_send_raw_sensor_measurements;
	adcs->adcs_send_raw_nadir_sensor = &adcs_send_raw_nadir_sensor;
	adcs->adcs_send_raw_sun_sensor = &adcs_send_raw_sun_sensor;
	adcs->adcs_send_raw_css = &adcs_send_raw_css;
	adcs->adcs_send_raw_magnetometer = &adcs_send_raw_magnetometer;
	adcs->adcs_send_raw_GPS_status = &adcs_send_raw_GPS_status;
	adcs->adcs_send_raw_GPS_time = &adcs_send_raw_GPS_time;
	adcs->adcs_send_raw_GPS_x = &adcs_send_raw_GPS_x;
	adcs->adcs_send_raw_GPS_y = &adcs_send_raw_GPS_y;
	adcs->adcs_send_raw_GPS_z = &adcs_send_raw_GPS_z;
	adcs->adcs_send_estimation_data = &adcs_send_estimation_data;
	adcs->adcs_send_IGRF_modelled_magnetic_field_vector = &adcs_send_IGRF_modelled_magnetic_field_vector;
	adcs->adcs_send_modelled_sun_vector = &adcs_send_modelled_sun_vector;
	adcs->adcs_send_estimated_gyro_bias = &adcs_send_estimated_gyro_bias;
	adcs->adcs_send_estimated_innovation_vector = &adcs_send_estimated_innovation_vector;
	adcs->adcs_send_quaternion_error_vector = &adcs_send_quaternion_error_vector;
	adcs->adcs_send_quaternion_covariance = &adcs_send_quaternion_covariance;
	adcs->adcs_send_angular_rate_covariance = &adcs_send_angular_rate_covariance;
	adcs->adcs_send_ACP_execution_state = &adcs_send_ACP_execution_state;
	adcs->adcs_send_ACP_execution_times = &adcs_send_ACP_execution_times;
	adcs->adcs_send_sgp4_orbit_parameters = &adcs_send_sgp4_orbit_parameters;
	adcs->adcs_send_curr_config = &adcs_send_curr_config;
	adcs->adcs_send_EDAC_and_latchup_counters = &adcs_send_EDAC_and_latchup_counters;
	adcs->adcs_send_start_up_mode = &adcs_send_start_up_mode;
	adcs->adcs_send_image_capture_and_save_operations_statuses = &adcs_send_image_capture_and_save_operations_statuses;
	adcs->adcs_send_file_info = &adcs_send_file_info;
	adcs->adcs_send_file_block_CRC = &adcs_send_file_block_CRC;
	adcs->adcs_send_data_block = &adcs_send_data_block;
	adcs->adcs_send_uploaded_program_status = &adcs_send_uploaded_program_status;
	adcs->adcs_send_flash_program_list = &adcs_send_flash_program_list;
}

