/*
 * Copyright (C) 2015  Brendan Bruner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * bbruner@ualberta.ca
 */
/**
 * @file comm_lpc_local
 * @author Brendan Bruner
 * @date Feb 25, 2015
 */

#include "comm.h"
#include "mock_comms_structs.h"

#define NO_ERROR 0 //CSP defines returning 0 as no errors.
#define RSSI_COUNT 10 //According to documentation this will be a ring buffer of size 10
#define HK_COUNT 8 //According to documentation this will be 8 structs

static int count = 0;

/************************************************************************/
/* Virtual method implementations										*/
/************************************************************************/
static comm_beacon_t beacon_mock( comm_t *comm )
{
	if( count++ < 45 )
	{
		return COMM_NOT_BEACONED;
	}
	if( count++ < 95 )
	{
		return COMM_IS_BEACONED;
	}
	count = 0;
	return COMM_NOT_BEACONED;
}

static int com_set_conf_mock( comm_t *comm, nanocom_conf_t *file )
{
	int i;

	comm->nanocom_conf_struct.do_rs = file->do_rs;
	comm->nanocom_conf_struct.do_random = file->do_random;
	comm->nanocom_conf_struct.do_vitebi = file->do_vitebi;
	comm->nanocom_conf_struct.tx_baud = file->tx_baud;
	comm->nanocom_conf_struct.rx_baud = file->rx_baud;
	comm->nanocom_conf_struct.tx_max_temp = file->tx_max_temp;
	comm->nanocom_conf_struct.preamble_length = file->preamble_length;
	comm->nanocom_conf_struct.morse_enable = file->morse_enable;
	comm->nanocom_conf_struct.morse_mode = file->morse_mode;
	comm->nanocom_conf_struct.morse_cycle = file->morse_cycle;
	comm->nanocom_conf_struct.morse_en_voltage = file->morse_en_voltage;
	comm->nanocom_conf_struct.morse_en_rx_count = file->morse_en_rx_count;
	comm->nanocom_conf_struct.morse_en_tx_count = file->morse_en_tx_count;
	comm->nanocom_conf_struct.morse_en_temp_a = file->morse_en_temp_a;
	comm->nanocom_conf_struct.morse_en_temp_b = file->morse_en_temp_b;
	comm->nanocom_conf_struct.morse_en_rssi = file->morse_en_rssi;
	comm->nanocom_conf_struct.morse_en_rf_err = file->morse_en_rf_err;
	comm->nanocom_conf_struct.morse_inter_delay = file->morse_inter_delay;
	comm->nanocom_conf_struct.morse_pospone = file->morse_pospone;
	comm->nanocom_conf_struct.morse_wpm = file->morse_wpm;
	for(i = 0; i<NANOCOM_CONF_MORSE_TEXT_LENGTH; i++)
	{
		comm->nanocom_conf_struct.morse_text[i] = file->morse_text[i];
	}
	comm->nanocom_conf_struct.morse_bat_level = file->morse_bat_level;
	comm->nanocom_conf_struct.hk_interval = file->hk_interval;

	return NO_ERROR;
}

static int com_get_conf_mock( comm_t *comm, nanocom_conf_t *file )
{
	int i;

	file->do_rs = comm->nanocom_conf_struct.do_rs;
	file->do_random = comm->nanocom_conf_struct.do_random;
	file->do_vitebi = comm->nanocom_conf_struct.do_vitebi;
	file->tx_baud = comm->nanocom_conf_struct.tx_baud;
	file->rx_baud = comm->nanocom_conf_struct.rx_baud;
	file->tx_max_temp = comm->nanocom_conf_struct.tx_max_temp;
	file->preamble_length = comm->nanocom_conf_struct.preamble_length;
	file->morse_enable = comm->nanocom_conf_struct.morse_enable;
	file->morse_mode = comm->nanocom_conf_struct.morse_mode;
	file->morse_cycle = comm->nanocom_conf_struct.morse_cycle;
	file->morse_en_voltage = comm->nanocom_conf_struct.morse_en_voltage;
	file->morse_en_rx_count = comm->nanocom_conf_struct.morse_en_rx_count;
	file->morse_en_tx_count = comm->nanocom_conf_struct.morse_en_tx_count;
	file->morse_en_temp_a = comm->nanocom_conf_struct.morse_en_temp_a;
	file->morse_en_temp_b = comm->nanocom_conf_struct.morse_en_temp_b;
	file->morse_en_rssi = comm->nanocom_conf_struct.morse_en_rssi;
	file->morse_en_rf_err = comm->nanocom_conf_struct.morse_en_rf_err;
	file->morse_inter_delay = comm->nanocom_conf_struct.morse_inter_delay;
	file->morse_pospone = comm->nanocom_conf_struct.morse_pospone;
	file->morse_wpm = comm->nanocom_conf_struct.morse_wpm;
	for(i = 0; i<NANOCOM_CONF_MORSE_TEXT_LENGTH; i++)
	{
		file->morse_text[i] = comm->nanocom_conf_struct.morse_text[i];
	}
	file->morse_bat_level = comm->nanocom_conf_struct.morse_bat_level;
	file->hk_interval = comm->nanocom_conf_struct.hk_interval;

	return NO_ERROR;
}

static int com_restore_conf_mock( comm_t *comm )
{
	int i;

	comm->nanocom_conf_struct.do_rs = (uint8_t) 0;
	comm->nanocom_conf_struct.do_random = (uint8_t) 0;
	comm->nanocom_conf_struct.do_vitebi = (uint8_t) 0;
	comm->nanocom_conf_struct.tx_baud = (uint8_t) 12;
	comm->nanocom_conf_struct.rx_baud = (uint8_t) 12;
	comm->nanocom_conf_struct.tx_max_temp = (int16_t) 0;
	comm->nanocom_conf_struct.preamble_length = (uint16_t) 50;
	comm->nanocom_conf_struct.morse_enable = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_mode = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_cycle = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_en_voltage = (uint8_t) 5;
	comm->nanocom_conf_struct.morse_en_rx_count = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_tx_count = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_temp_a = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_temp_b = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_rssi = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_rf_err = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_inter_delay = (uint16_t) 10;
	comm->nanocom_conf_struct.morse_pospone = (uint16_t) 10;
	comm->nanocom_conf_struct.morse_wpm = (uint8_t) 10;
	for(i = 0; i<NANOCOM_CONF_MORSE_TEXT_LENGTH; i++)
	{
		comm->nanocom_conf_struct.morse_text[i] = (uint8_t) 0;
	}
	comm->nanocom_conf_struct.morse_bat_level = (uint16_t) 500;
	comm->nanocom_conf_struct.hk_interval = (uint16_t) 10;

	return NO_ERROR;

}

static int com_get_status_mock( comm_t *comm, nanocom_data_t *data )
{

	data-> bit_corr_tot = (uint32_t) 0;
	data-> byte_corr_tot = (uint32_t) 0;
	data-> rx = (uint32_t) 0;
	data-> rx_err = (uint32_t)0;
	data-> tx = (uint32_t) 0;
	data-> last_temp_a = (int16_t) 0;
	data-> last_temp_b = (int16_t) 0;
	data-> last_rssi = (int16_t) 0;
	data-> last_rferr = (int16_t) 0;
	data-> last_batt_volt = (uint16_t) 0;
	data-> last_txcurrent = (uint16_t) 0;
	data-> bootcount = (uint32_t) 0;

	return NO_ERROR;
}

static int com_get_log_rssi_mock( comm_t *comm, nanocom_rssi_t * data, uint8_t * count )
{
	int i;

	for(i = 0; i<RSSI_COUNT; i++)
	{
		data[i].time = (uint32_t) 0;
		data[i].rssi = (int16_t) 0;
		data[i].rferr = (int16_t) 0;
	}

	*count = (uint8_t) RSSI_COUNT;
	return NO_ERROR;
}

static int com_get_hk_mock(comm_t *comm, nanocom_hk_t *data, uint8_t *count )
{
	int i;

	for(i = 0; i<HK_COUNT; i++)
	{
		data[i].time = (uint32_t) 0;;
		data[i].temp_a = (int16_t) 0;
		data[i].temp_b = (int16_t) 0;
		data[i].batt_volt = (uint16_t) 0;
	}

	*count = (uint8_t) HK_COUNT;
	return NO_ERROR;
}

static uint8_t poll_telecommand( comm_t *comm, telecommand_packet_t *tc)
{
	return NO_TELECOMMAND;
}

static uint8_t transmit( comm_t *comm, uint8_t *data, uint16_t size )
{
	return 0;
}

/************************************************************************/
/* Initialization 														*/
/************************************************************************/

static void initialize_conf_struct( comm_t *comm)
{
	int i;

	//For nancom_conf_struct
	comm->nanocom_conf_struct.do_rs = (uint8_t) 0;
	comm->nanocom_conf_struct.do_random = (uint8_t) 0;
	comm->nanocom_conf_struct.do_vitebi = (uint8_t) 0;
	comm->nanocom_conf_struct.tx_baud = (uint8_t) 12;
	comm->nanocom_conf_struct.rx_baud = (uint8_t) 12;
	comm->nanocom_conf_struct.tx_max_temp = (int16_t) 0;
	comm->nanocom_conf_struct.preamble_length = (uint16_t) 50;
	comm->nanocom_conf_struct.morse_enable = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_mode = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_cycle = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_en_voltage = (uint8_t) 5;
	comm->nanocom_conf_struct.morse_en_rx_count = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_tx_count = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_temp_a = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_temp_b = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_rssi = (uint8_t) 10;
	comm->nanocom_conf_struct.morse_en_rf_err = (uint8_t) 0;
	comm->nanocom_conf_struct.morse_inter_delay = (uint16_t) 10;
	comm->nanocom_conf_struct.morse_pospone = (uint16_t) 10;
	comm->nanocom_conf_struct.morse_wpm = (uint8_t) 10;
	for(i = 0; i<NANOCOM_CONF_MORSE_TEXT_LENGTH; i++)
	{
		comm->nanocom_conf_struct.morse_text[i] = (uint8_t) 0;
	}
	comm->nanocom_conf_struct.morse_bat_level = (uint16_t) 500;
	comm->nanocom_conf_struct.hk_interval = (uint16_t) 10;

}

void initialize_comm_lpc_local( comm_t *comm )
{
	initialize_conf_struct( comm );
	comm->data = 0;
	comm->beacon = &beacon_mock;
	comm->com_set_conf = &com_set_conf_mock;
	comm->com_get_conf = &com_get_conf_mock;
	comm->com_restore_conf = &com_restore_conf_mock;
	comm->com_get_status = &com_get_status_mock;
	comm->com_get_log_rssi = &com_get_log_rssi_mock;
	comm->com_get_hk = &com_get_hk_mock;
	comm->poll_telecommand = &poll_telecommand;
	comm->transmit = &transmit;
}
