/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include <stdio.h>
#include "i2c.h"

/* Global variables to keep track of transfers */
__IO FlagStatus complete_S;
__IO FlagStatus complete_M;

/* Transmit and receive buffers for Master and Slave */
char Slave_Buf[BUFFER_SIZE];

/*********************************************************************//**
 * @brief		Interrupt Service Routine for Slave Peripheral
 * 				Assigns Handle to interrupt handler and sets flag
 * 				upon completion
 * @param[in]	None
 * @return		None
 **********************************************************************/
void I2C1_IRQHandler() {
	//I2C1_StdIntHandler();
	/*I2C_SlaveHandler(I2C_SLAVE);
	if(I2C_SlaveTransferComplete(I2C_SLAVE)) {
		complete_S = SET;
	}*/
}

void MasterCallback(void)
{
	complete_M = SET;
}



/*********************************************************************//**
 * @brief		I2C initialization function
 * @param[in]	None
 * @return		None
 **********************************************************************/
void setupI2C ()
{
	I2C_OWNSLAVEADDR_CFG_Type SlaveAddrConfig;
	PINSEL_CFG_Type PinCfg;

	complete_M = RESET;

	/* Init I2C pins */
	PinCfg.Portnum   = 0;
	PinCfg.Pinnum    = 0;
	PinCfg.Funcnum   = PINSEL_FUNC_3;
	PinCfg.OpenDrain = PINSEL_PINMODE_NORMAL;
	PinCfg.Pinmode   = PINSEL_PINMODE_PULLUP;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum    = 1;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Funcnum   = PINSEL_FUNC_3;
	PinCfg.Pinnum 	 = 10;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum 	 = 11;
	PINSEL_ConfigPin(&PinCfg);
/*
	SlaveAddrConfig.GeneralCallState = DISABLE;
	SlaveAddrConfig.SlaveAddrChannel = 0;
	SlaveAddrConfig.SlaveAddrMaskValue = 0xFF;
	SlaveAddrConfig.SlaveAddr_7bit = I2C_SLAVE_ADDR;
	I2C_SetOwnSlaveAddr(I2C_SLAVE, &SlaveAddrConfig);
*/
	/* Initialize Master and Slave I2C peripherals */
	I2C_Init(I2C_SLAVE, I2C_CLK_RATE);


	/* Set up interrupts
	 * Note: Master has higher priority
	 */
	NVIC_DisableIRQ(I2C1_IRQn);
	NVIC_SetPriority(I2C1_IRQn, ((0x00<<3)|0x01));

	/*Enable Master and Slave I2C peripherals */
	I2C_Cmd(I2C_SLAVE, ENABLE);

}

void i2c_send( uint8_t *data, uint8_t length, uint8_t addr )
{
	I2C_M_SETUP_Type master_tx = { 	.sl_addr7bit         = (uint8_t) addr,
									.tx_data             = data,
									.tx_length           = (uint8_t) length,
									.tx_count            = 0,
									.retransmissions_max = 10,
									.callback = MasterCallback };

    I2C_MasterTransferData(I2C_SLAVE, &master_tx, I2C_TRANSFER_INTERRUPT );
    while( complete_M == RESET );
    complete_M = RESET;
}
/*********************************************************************//**
 * @brief		Task to test Master-Slave Communication
 * @param[in]	None
 * @return		None
 **********************************************************************/
void I2C_test ( void *pvParameters )
{
	I2C_S_SETUP_Type SlaveConfig;
	uint32_t i;
	char buf;
	/* this loop prevents the task from ending */
	for (i = 0 ;i < BUFFER_SIZE ; i++) {
		/* MASTER SEND DATA TO SLAVE -------------------------------------------------------- */
		/* Force complete flag for the first time of running */
		complete_S =  RESET;

		printf(" slave...\n");
		SlaveConfig.tx_data = NULL;
		SlaveConfig.tx_length = 0;
		SlaveConfig.rx_data = Slave_Buf;
		SlaveConfig.rx_length = sizeof(Slave_Buf);
		I2C_SlaveTransferData(I2C_SLAVE, &SlaveConfig, I2C_TRANSFER_INTERRUPT);
		/* Wait until complete */
		while ((complete_S == RESET));
		printf("Received: %s", Slave_Buf);

		vTaskDelay ( 1000 / portTICK_RATE_MS );
	}
}

#define LCD_SETCURS_LEN	 4		/* Number of packets used in setCursor	*/
#define LCD_CLEAR_LEN	 3		/* Number of packets used to clear		*/
#define LCD_MAX_PACKET_LEN	17	/* The maximum number of packets the LCD*/
#define LCD_LINE1_END	 0x0F	/* End of line 1 0x00 - 0x0F			*/
#define LCD_LINE2_END	 0x4F	/* End of line 2 0x40 - 0x4F			*/
#define LCD_LINE2_OFFSET 0x30	/* Off set added to cursor position when*/
#define LCD_PREFIX			0xFE	/* Prefix for LCD command			*/
#define LCD_SET_CURSOR		0x45	/* Set cursor command				*/
#define LCD_UNDERLINE_OFF	0x48	/* Underline cursor off				*/
#define LCD_BLINKING_OFF	0x4C	/* Blinking cursor off				*/
#define LCD_CLEAR_SCREEN	0x51	/* Clear LCD screen					*/
#define LCD_SET_CONTRAST	0x52	/* Set contrast of LCD screen		*/
#define LCD_SET_BACKLIGHT	0x53	/* Set backlight brightness			*/
#define LCD_CONTRAST		40		/* LCD contrast level				*/
#define LCD_BRIGHTNESS		8		/* LCD brightness level				*/
#define LCD_SLAVEADDRESS	0x28	/* Slave address of the LCD screen		*/
#define CursorPos(a) (((a) > LCD_LINE1_END)\
					 ? ((a) + LCD_LINE2_OFFSET)\
					 : (a))
void lcd_init( void )
{
	uint8_t LCD_initCodes[10];

	LCD_initCodes[0] = LCD_PREFIX;
	LCD_initCodes[1] = LCD_SET_CONTRAST;
	LCD_initCodes[2] = LCD_CONTRAST;

	/* Set the brightness of the LCD */
	LCD_initCodes[3] = LCD_PREFIX;
	LCD_initCodes[4] = LCD_SET_BACKLIGHT;
	LCD_initCodes[5] = LCD_BRIGHTNESS;

	/* Turn blinking, and underlined cursor off */
	LCD_initCodes[6] = LCD_PREFIX;
	LCD_initCodes[7] = LCD_BLINKING_OFF;
	LCD_initCodes[8] = LCD_PREFIX;
	LCD_initCodes[9] = LCD_UNDERLINE_OFF;

	i2c_send( LCD_initCodes, 10, LCD_SLAVEADDRESS );
}

void lcd_seek( uint8_t position )
{
	uint8_t LCD_cursorCodes[LCD_SETCURS_LEN-1];

	/* Adjust position to place cursor where it should go */
	position = CursorPos(position);

	/* Set up the packets to send to the LCD which change the position */
	LCD_cursorCodes[0] = LCD_PREFIX;
	LCD_cursorCodes[1] = LCD_SET_CURSOR;
	LCD_cursorCodes[2] = position;

	i2c_send( LCD_cursorCodes, LCD_SETCURS_LEN-1, LCD_SLAVEADDRESS );
}

void lcd_write( uint8_t *data, uint8_t length )
{
	i2c_send( data, length, LCD_SLAVEADDRESS );
}

void lcd_clear( void )
{
	uint8_t LCD_clearCodes[2];

	LCD_clearCodes[0] = LCD_PREFIX;
	LCD_clearCodes[1] = LCD_CLEAR_SCREEN;

	i2c_send( LCD_clearCodes, 2, LCD_SLAVEADDRESS );
}

